@@ -224,6 +224,7 @@ typedef struct
 #define ACS_ACR33U_A2_3SAM_ICC_READER	0x072f8302
 #define ACS_ACR33U_A3_3SAM_ICC_READER	0x072f8307
 #define ACS_ACR33U_4SAM_ICC_READER		0x072f8301
+#define ACS_ACR39U_SAM_ICC_READER		0x072fb104
 #define ACS_ACR83U						0x072f90d2
 #define ACS_ACR85_PINPAD_READER_ICC		0x072f8306
 #define ACS_ACR85_PINPAD_READER_PICC	0x072f2210
@@ -236,6 +237,7 @@ typedef struct
 #define ACS_ACR1222_1SAM_DUAL_READER	0x072f1280	// ACR1222U-C3
 #define ACS_ACR1222_DUAL_READER			0x072f2207	// ACR1222U-C6
 #define ACS_ACR1222_3S_PICC_READER		0x072f2206	// ACR1222L-D1
+#define ACS_ACR1251_1S_CL_READER		0x072f221a
 #define ACS_ACR128U						0x072f2100
 #define ACS_ACR1281_DUAL_READER_QPBOC	0x072f2208
 #define ACS_ACR1281_PICC_READER_BSI		0x072f0901
@@ -243,6 +245,9 @@ typedef struct
 #define ACS_ACR1281_1S_DUAL_READER		0x072f2224
 #define ACS_ACR1281_2S_CL_READER		0x072f2215
 #define ACS_ACR1281_1S_PICC_READER		0x072f2220
+#define ACS_ACR1283_4S_CL_READER		0x072f2213
+#define ACS_ACR1283_CL_READER			0x072f222C
+#define ACS_ACR1283U_FW_UPGRADE			0x072f220C
 #define ACS_AET62_PICC_READER			0x072f0102
 #define ACS_AET62_1SAM_PICC_READER		0x072f0103
 #define ACS_APG8201						0x072f8201
