@@ -24,6 +24,10 @@
 
 #define __CCID_USB__
 
+#ifdef __APPLE__
+#include <IOKit/usb/IOUSBLib.h>
+#endif
+
 #include <stdio.h>
 #include <string.h>
 #include <errno.h>
@@ -106,7 +110,53 @@ static unsigned int *get_data_rates(unsigned int reader_index,
 	struct usb_device *dev, int num);
 
 #ifdef __APPLE__
+
+#if defined (kIOUSBInterfaceInterfaceID220)
+#define usb_interface_t	IOUSBInterfaceInterface220
+#elif defined (kIOUSBInterfaceInterfaceID197)
+#define usb_interface_t	IOUSBInterfaceInterface197
+#elif defined (kIOUSBInterfaceInterfaceID190)
+#define usb_interface_t	IOUSBInterfaceInterface190
+#elif defined (kIOUSBInterfaceInterfaceID182)
+#define usb_interface_t	IOUSBInterfaceInterface182
+#else
+#define usb_interface_t	IOUSBInterfaceInterface
+#endif
+
+#if defined (kIOUSBDeviceInterfaceID197)
+#define usb_device_t	IOUSBDeviceInterface197
+#elif defined (kIOUSBDeviceInterfaceID187)
+#define usb_device_t	IOUSBDeviceInterface187
+#elif defined (kIOUSBDeviceInterfaceID182)
+#define usb_device_t	IOUSBDeviceInterface182
+#else
+#define usb_device_t	IOUSBDeviceInterface
+#endif
+
+struct darwin_dev_handle {
+	usb_device_t **device;
+	usb_interface_t **interface;
+	int open;
+
+	int num_endpoints;
+	unsigned char *endpoint_addrs;
+};
+
+struct usb_dev_handle {
+	int fd;
+
+	struct usb_bus *bus;
+	struct usb_device *device;
+
+	int config;
+	int interface;
+	int altsetting;
+
+	void *impl_info;
+};
+
 static void *CardDetectionThread(void *pParam);
+static int GetPipeRef(struct darwin_dev_handle *device, int ep);
 #endif
 
 /* ne need to initialize to 0 since it is static */
@@ -274,6 +324,22 @@ status_t OpenUSBByName(unsigned int reader_index, /*@null@*/ char *device)
 			/* dirname was just a temporary variable */
 			dirname = NULL;
 		}
+
+		/* format usb:%04x/%04x:libudev:%d:%s
+		 * with %d set to
+		 * 01 (or whatever the interface number is)
+		 * and %s set to
+		 * /dev/bus/usb/008/004
+		 */
+		if ((dirname = strstr(device, "libudev:")) != NULL)
+		{
+			/* convert the interface number */
+			interface_number = atoi(dirname + 8 /* "libudev:" */);
+			DEBUG_COMM2("interface_number: %d", interface_number);
+
+			/* dirname was just a temporary variable */
+			dirname = NULL;
+		}
 	}
 #endif
 
@@ -401,7 +467,8 @@ status_t OpenUSBByName(unsigned int reader_index, /*@null@*/ char *device)
 						// Simulate ACR1281 Dual Reader (composite device) as multi-slot reader
 						else if ((ACS_ACR1281_DUAL_READER_QPBOC == readerID) ||
 							(ACS_ACR1281_DUAL_READER_BSI == readerID) ||
-							(ACS_ACR1281_1S_PICC_READER == readerID))
+							(ACS_ACR1281_1S_PICC_READER == readerID) ||
+							(ACS_ACR1251_1S_CL_READER == readerID))
 						{
 							// the CCID interfaces are 0 and 1
 							interface_number = static_interface - 1;
@@ -784,6 +851,14 @@ again:
 
 					// Initialize isSamSlot
 					usbDevice[reader_index].ccid.isSamSlot = FALSE;
+
+					// The 2nd interface (composite device) is a SAM slot
+					if ((readerID == ACS_ACR1281_1S_PICC_READER) ||
+						(readerID == ACS_ACR1251_1S_CL_READER))
+					{
+						if (interface == 1)
+							usbDevice[reader_index].ccid.isSamSlot = TRUE;
+					}
 #ifdef __APPLE__
 					// Initialize terminated flag to false
 					usbDevice[reader_index].terminated = FALSE;
@@ -1056,26 +1131,37 @@ status_t CloseUSB(unsigned int reader_index)
 	/* release the allocated ressources for the last slot only */
 	if (0 == *usbDevice[reader_index].nb_opened_slots)
 	{
+#ifdef __APPLE__
+		struct darwin_dev_handle *device;
+		int pipeRef;
+#endif
 		DEBUG_COMM("Last slot closed. Release resources");
 
 		/* reset so that bSeq starts at 0 again */
 		if (DriverOptions & DRIVER_OPTION_RESET_ON_CLOSE)
 			(void)usb_reset(usbDevice[reader_index].handle);
-
-		(void)usb_release_interface(usbDevice[reader_index].handle,
-			usbDevice[reader_index].interface);
-		(void)usb_close(usbDevice[reader_index].handle);
 #ifdef __APPLE__
 		DEBUG_INFO3("Terminating thread: %s/%s",
 			usbDevice[reader_index].dirname, usbDevice[reader_index].filename);
 
 		// Terminate thread
 		*usbDevice[reader_index].pTerminated = TRUE;
+
+		// Abort interrupt pipe
+		device = (struct darwin_dev_handle *) usbDevice[reader_index].handle->impl_info;
+		pipeRef = GetPipeRef(device, usbDevice[reader_index].interrupt);
+		(*(device->interface))->AbortPipe(device->interface, pipeRef);
+
+		// Wait thread
 		pthread_join(usbDevice[reader_index].hThread, NULL);
 
 		// Free bStatus lock
 		pthread_mutex_destroy(usbDevice[reader_index].ccid.pbStatusLock);
 #endif
+		(void)usb_release_interface(usbDevice[reader_index].handle,
+			usbDevice[reader_index].interface);
+		(void)usb_close(usbDevice[reader_index].handle);
+
 		// Free array of bStatus
 		free(usbDevice[reader_index].ccid.bStatus);
 
@@ -1088,7 +1174,7 @@ status_t CloseUSB(unsigned int reader_index)
 	usbDevice[reader_index].dirname = NULL;
 	usbDevice[reader_index].filename = NULL;
 	usbDevice[reader_index].interface = 0;
-	usbDevice[reader_index].ccid.bStatus = NULL;		// Array of bStatus
+	usbDevice[reader_index].ccid.bStatus = NULL;	// Array of bStatus
 
 	return STATUS_SUCCESS;
 } /* CloseUSB */
@@ -1342,6 +1428,15 @@ int ControlUSB(int reader_index, int requesttype, int request, int value,
 		request, value, usbDevice[reader_index].interface, (char *)bytes, size,
 		usbDevice[reader_index].ccid.readTimeout * 1000);
 
+	if (ret < 0)
+	{
+		DEBUG_CRITICAL4("control failed (%s/%s): %s",
+			usbDevice[reader_index].dirname, usbDevice[reader_index].filename,
+			strerror(errno));
+
+		return ret;
+	}
+
 	if (requesttype & 0x80)
 		DEBUG_XXD("receive: ", bytes, ret);
 
@@ -1429,4 +1524,16 @@ static void *CardDetectionThread(void *pParam)
 	DEBUG_INFO2("Exit: Reader: %d", reader_index);
 	return ((void *) 0);
 }
+
+// Get pipe reference from endpoint number
+static int GetPipeRef(struct darwin_dev_handle *device, int ep)
+{
+	int i;
+
+	for (i = 0; i < device->num_endpoints; i++)
+		if (device->endpoint_addrs[i] == ep)
+			return i + 1;
+
+	return -1;
+}
 #endif
